local waitTable = {};
local waitFrame = nil;

function serializeTable(val, name, skipnewlines, depth)
    skipnewlines = skipnewlines or false
    depth = depth or 0

    -- local tmp = string.rep(" ", depth)
    local tmp = ""

    if name then tmp = tmp .. name .. "=" end

    if type(val) == "table" then
        -- tmp = tmp .. (not skipnewlines and "\n" or "")

        for k, v in pairs(val) do
            tmp =  tmp .. serializeTable(v, k, skipnewlines, depth + 1) .. (not skipnewlines and "\n" or "")
        end

        tmp = tmp .. string.rep(" ", depth)
    elseif type(val) == "number" then
        tmp = tmp .. tostring(val)
    elseif type(val) == "string" then
        tmp = tmp .. val
    elseif type(val) == "boolean" then
        tmp = tmp .. (val and "true" or "false")
    else
        tmp = tmp .. "\"[inserializeable datatype:" .. type(val) .. "]\""
    end

    return tmp
end

function magiclines(s)
        if s:sub(-1)~="\n" then s=s.."\n" end
        return s:gmatch("(.-)\n")
end

function unserializeString(val)
    autoroller_table = {}
    for line in magiclines(val) do
        local item, roll = line:match("([^=]+)=([^=]+)")
        autoroller_table[item] = roll
    end
    return true
end

function KethoEditBox_Show(text)
    if not KethoEditBox then
        local f = CreateFrame("Frame", "KethoEditBox", UIParent, "DialogBoxFrame")

        f:SetPoint("CENTER")
        f:SetSize(600, 500)

        f:SetBackdrop({
            bgFile = "Interface\\DialogFrame\\UI-DialogBox-Background",
            edgeFile = "Interface\\PVPFrame\\UI-Character-PVP-Highlight", -- this one is neat
            edgeSize = 16,
            insets = { left = 8, right = 6, top = 8, bottom = 8 },
        })
        f:SetBackdropBorderColor(0, .44, .87, 0.5) -- darkblue

        -- Movable
        f:SetMovable(true)
        f:SetClampedToScreen(true)
        f:SetScript("OnMouseDown", function(self, button)
            if button == "LeftButton" then
                self:StartMoving()
            end
        end)
        f:SetScript("OnMouseUp", f.StopMovingOrSizing)

        -- ScrollFrame
        local sf = CreateFrame("ScrollFrame", "KethoEditBoxScrollFrame", KethoEditBox, "UIPanelScrollFrameTemplate")
        sf:SetPoint("LEFT", 16, 0)
        sf:SetPoint("RIGHT", -32, 0)
        sf:SetPoint("TOP", 0, -16)
        sf:SetPoint("BOTTOM", KethoEditBoxButton, "TOP", 0, 0)

        -- EditBox
        local eb = CreateFrame("EditBox", "KethoEditBoxEditBox", KethoEditBoxScrollFrame)
        eb:SetSize(sf:GetSize())
        eb:SetMultiLine(true)
        eb:SetAutoFocus(false) -- dont automatically focus
        eb:SetFontObject("ChatFontNormal")
        eb:SetScript("OnEscapePressed", function() f:Hide() end)
        sf:SetScrollChild(eb)


        -- Resizable
        f:SetResizable(true)
        f:SetMinResize(150, 100)

        local rb = CreateFrame("Button", "KethoEditBoxResizeButton", KethoEditBox)
        rb:SetPoint("BOTTOMRIGHT", -6, 7)
        rb:SetSize(16, 16)

        rb:SetNormalTexture("Interface\\ChatFrame\\UI-ChatIM-SizeGrabber-Up")
        rb:SetHighlightTexture("Interface\\ChatFrame\\UI-ChatIM-SizeGrabber-Highlight")
        rb:SetPushedTexture("Interface\\ChatFrame\\UI-ChatIM-SizeGrabber-Down")

        rb:SetScript("OnMouseDown", function(self, button)
            if button == "LeftButton" then
                f:StartSizing("BOTTOMRIGHT")
                self:GetHighlightTexture():Hide() -- more noticeable
            end
        end)
        rb:SetScript("OnMouseUp", function(self, button)
            f:StopMovingOrSizing()
            self:GetHighlightTexture():Show()
            eb:SetWidth(sf:GetWidth())
        end)
        f:Show()
    end

    if text then
        KethoEditBoxEditBox:SetText(text)
    else
        KethoEditBoxEditBox:SetText("")
    end

    KethoEditBoxButton:HookScript("OnClick", function(self)
        local my_string = KethoEditBoxEditBox:GetText()
        foo = unserializeString(my_string)
    end)
    KethoEditBox:Show()
end


function dwgautoroller_chatfilter_allrolls(frame, event, message, sender, ...)
    if string.find(message, "You have selected") or string.find(message, "has selected") or string.find(message, "Need Roll - ") or string.find(message, "Greed Roll - ") then
        return true
    end
    return false, message, sender, ... -- don't hide this message
end

function dwgautoroller_chatfilter(frame, event, message, sender, ...)
    for item_name, rolltype in pairs(autoroller_table) do
        if string.find(message, "You receive loot") then
            return false, message, sender, ... -- don't hide this message
        end
        if string.find(message, item_name) then
            -- Returning true here hides the message
            return true
        end
    end
    return false, message, sender, ... -- don't hide this message
end

function dwgautoroller__wait(delay, func, ...)
    if (type(delay) ~= "number" or type(func) ~= "function") then
        return false;
    end
    if (waitFrame == nil) then
        waitFrame = CreateFrame("Frame", "WaitFrame", UIParent);
        waitFrame:SetScript("onUpdate", function(self, elapse)
            local count = #waitTable;
            local i = 1;
            while (i <= count) do
                local waitRecord = tremove(waitTable, i);
                local d = tremove(waitRecord, 1);
                local f = tremove(waitRecord, 1);
                local p = tremove(waitRecord, 1);
                if (d > elapse) then
                    tinsert(waitTable, i, { d - elapse, f, p });
                    i = i + 1;
                else
                    count = count - 1;
                    f(unpack(p));
                end
            end
        end);
    end
    tinsert(waitTable, { delay, func, { ... } });
    return true;
end

autoroller_table = {}
local frame = CreateFrame("FRAME"); -- Need a frame to respond to events
frame:RegisterEvent("ADDON_LOADED"); -- Fired when saved variables are loaded
frame:RegisterEvent("PLAYER_LOGOUT"); -- Fired when about to log out

local lootid_rolled_on = {} -- Empty table for loot we have rolled on.

local function setup_mcall(rolltype)
    autoroller_table["Shadowstrike"] = rolltype
    autoroller_table["Crimson Shocker"] = rolltype
    autoroller_table["Manastorm Leggings"] = rolltype
    autoroller_table["Sorcerous Dagger"] = rolltype
    autoroller_table["Earthshaker"] = rolltype
    autoroller_table["Eskhandar's Right Claw"] = rolltype
    autoroller_table["Medalion of Steadfast Might"] = rolltype
    autoroller_table["Seal of the Archmagus"] = rolltype
    autoroller_table["Deep Earth Spaulders"] = rolltype
    autoroller_table["Fire Runed Grimoire"] = rolltype
    autoroller_table["Fireproof Cloak"] = rolltype
    autoroller_table["Gloves of the Hypnotic Flame"] = rolltype
    autoroller_table["Core Forged Greaves"] = rolltype
    autoroller_table["Band of Sulfuras"] = rolltype
    autoroller_table["Essence of the Pure Flame"] = rolltype
    autoroller_table["Shard of the Flame"] = rolltype
    autoroller_table["Heavy Dark Iron Ring"] = rolltype
    autoroller_table["Helm of the Lifegiver"] = rolltype
    autoroller_table["Blastershot Launcher"] = rolltype
    autoroller_table["Fireguard Shoulders"] = rolltype
    autoroller_table["Sabatons of the Flamewalker"] = rolltype
    autoroller_table["Finkle's Lava Dredger"] = rolltype
    autoroller_table["Sash of Whispered Secrets"] = rolltype
    autoroller_table["Crown of Destruction"] = rolltype
    autoroller_table["Dragon's Blood Cape"] = rolltype
    autoroller_table["Malistar's Defender"] = rolltype
    autoroller_table["Flamewaker Legplates"] = rolltype
    autoroller_table["Sabatons of Might"] = rolltype
    autoroller_table["Gauntlets of Might"] = rolltype
    autoroller_table["Breastplate of Might"] = rolltype
    autoroller_table["Legplates of Might"] = rolltype
    autoroller_table["Helm of Might"] = rolltype
    autoroller_table["Pauldrons of Might"] = rolltype
    autoroller_table["Earthfury Boots"] = rolltype
    autoroller_table["Earthfury Vestments"] = rolltype
    autoroller_table["Earthfury Epaulets"] = rolltype
    autoroller_table["Earthfury Gauntlets"] = rolltype
    autoroller_table["Earthfury Helmet"] = rolltype
    autoroller_table["Earthfury Legguards"] = rolltype
    autoroller_table["Giantstalker's Boots"] = rolltype
    autoroller_table["Giantstalker's Breastplate"] = rolltype
    autoroller_table["Giantstalker's Epaulets"] = rolltype
    autoroller_table["Giantstalker's Gloves"] = rolltype
    autoroller_table["Giantstalker's Helmet"] = rolltype
    autoroller_table["Giantstalker's Leggings"] = rolltype
    autoroller_table["Cenarion Boots"] = rolltype
    autoroller_table["Cenarion Vestments"] = rolltype
    autoroller_table["Cenarion Gloves"] = rolltype
    autoroller_table["Cenarion Helm"] = rolltype
    autoroller_table["Cenarion Leggings"] = rolltype
    autoroller_table["Cenarion Spaulders"] = rolltype
    autoroller_table["Nightslayer Boots"] = rolltype
    autoroller_table["Nightslayer Chestpiece"] = rolltype
    autoroller_table["Nightslayer Cover"] = rolltype
    autoroller_table["Nightslayer Gloves"] = rolltype
    autoroller_table["Nightslayer Pants"] = rolltype
    autoroller_table["Nightslayer Shoulder Pads"] = rolltype
    autoroller_table["Boots of Prophecy"] = rolltype
    autoroller_table["Circlet of Prophecy"] = rolltype
    autoroller_table["Gloves of Prophecy"] = rolltype
    autoroller_table["Pants of Prophecy"] = rolltype
    autoroller_table["Mantle of Prophecy"] = rolltype
    autoroller_table["Robes of Prophecy"] = rolltype
    autoroller_table["Arcanist Crown"] = rolltype
    autoroller_table["Arcanist Boots"] = rolltype
    autoroller_table["Arcanist Gloves"] = rolltype
    autoroller_table["Arcanist Leggings"] = rolltype
    autoroller_table["Arcanist Mantle"] = rolltype
    autoroller_table["Arcanist Robes"] = rolltype
    autoroller_table["Felheart Gloves"] = rolltype
    autoroller_table["Felheart Pants"] = rolltype
    autoroller_table["Felheart Robes"] = rolltype
    autoroller_table["Felheart Shoulder Pads"] = rolltype
    autoroller_table["Felheart Horns"] = rolltype
    autoroller_table["Felheart Slippers"] = rolltype
    autoroller_table["Choker of Enlightenment"] = rolltype
    autoroller_table["Wristguards of Stability"] = rolltype
    autoroller_table["Aurastone Hammer"] = rolltype
    autoroller_table["Striker's Mark"] = rolltype
    autoroller_table["Drillborer Disk"] = rolltype
    autoroller_table["Gutgore Ripper"] = rolltype
    autoroller_table["Aged Core Leather Gloves"] = rolltype
    autoroller_table["Flameguard Gauntlets"] = rolltype
    autoroller_table["Wild Growth Spaulders"] = rolltype
    autoroller_table["Wristguards of True Flight"] = rolltype
    autoroller_table["Cloak of the Shrouded Mists"] = rolltype
    autoroller_table["Netherwind Pants"] = rolltype
    autoroller_table["Stormrage Legguards"] = rolltype
    autoroller_table["Nemesis Leggings"] = rolltype
    autoroller_table["Leggings of Transcendence"] = rolltype
    autoroller_table["Legplates of Wrath"] = rolltype
    autoroller_table["Bloodfang Pants"] = rolltype
    autoroller_table["Dragonstalker's Legguards"] = rolltype
    autoroller_table["Legplates of Ten Storms"] = rolltype
    autoroller_table["Brutality Blade"] = rolltype
    autoroller_table["Robe of Volatile Power"] = rolltype
    autoroller_table["Salamander Scale Pants"] = rolltype
    autoroller_table["Staff of Dominance"] = rolltype
    autoroller_table["Mana Igniting Cord"] = rolltype
    autoroller_table["Obsidian Edged Blade"] = rolltype
    autoroller_table["Quick Strike Ring"] = rolltype
    autoroller_table["Talisman of Ephemeral Power"] = rolltype
    autoroller_table["Cauterising Band"] = rolltype
    autoroller_table["Core Hound Tooth"] = rolltype
    autoroller_table["The Eye of Divinity"] = rolltype
    autoroller_table["Ancient Petrified Leaf"] = rolltype
    autoroller_table["Ring of Spell Power"] = rolltype
    autoroller_table["Azuresong Mageblade"] = rolltype
    autoroller_table["Band of Accuria"] = rolltype
    autoroller_table["Bonereavers Edge"] = rolltype
    autoroller_table["Choker of the Fire Lord"] = rolltype
    autoroller_table["Onslaught Girdle"] = rolltype
    autoroller_table["Perdition's Blade"] = rolltype
    autoroller_table["Spinal Reaper"] = rolltype
    autoroller_table["Bindings of the Windseeker"] = rolltype
    autoroller_table["Bracers of Might"] = rolltype
    autoroller_table["Belt of Might"] = rolltype
    autoroller_table["Earthfury Belt"] = rolltype
    autoroller_table["Earthfury Bracers"] = rolltype
    autoroller_table["Giantstalker's Belt"] = rolltype
    autoroller_table["Giantstalker's Bracers"] = rolltype
    autoroller_table["Cenarion Belt"] = rolltype
    autoroller_table["Cenarion Bracers"] = rolltype
    autoroller_table["Nightslayer Belt"] = rolltype
    autoroller_table["Nightslayer Bracelets"] = rolltype
    autoroller_table["Girdle of Prophecy"] = rolltype
    autoroller_table["Vambraces of Prophecy"] = rolltype
    autoroller_table["Arcanist Belt"] = rolltype
    autoroller_table["Arcanist Bindings"] = rolltype
    autoroller_table["Felheart Belt"] = rolltype
    autoroller_table["Felheart Bracers"] = rolltype
    autoroller_table["Eye of Sulfuras"] = rolltype
    autoroller_table["Lava Core"] = rolltype
    autoroller_table["Fiery Core"] = rolltype
    autoroller_table["Blood of the Mountain"] = rolltype
    print("Now rolling " .. rolltype .. " on all MC items")
end

local function setup_bwlall(rolltype)
    autoroller_table["Shadowstrike"] = rolltype
    autoroller_table["The Black Book"] = rolltype
    autoroller_table["Rune of Metamorphosis"] = rolltype
    autoroller_table["Venomous Totem"] = rolltype
    autoroller_table["Black Ash Robe"] = rolltype
    autoroller_table["Emberweave Leggings"] = rolltype
    autoroller_table["Dragon's Touch"] = rolltype
    autoroller_table["Interlaced Shadow Jerkin"] = rolltype
    autoroller_table["Gloves of Rapid Evolution"] = rolltype
    autoroller_table["Arcane Infused Gem"] = rolltype
    autoroller_table["Spineshatter"] = rolltype
    autoroller_table["Black Brood Pauldrons"] = rolltype
    autoroller_table["Claw of the Black Drake"] = rolltype
    autoroller_table["Natural Alignment Crystal"] = rolltype
    autoroller_table["Primalist Linked Legguards"] = rolltype
    autoroller_table["Shadow Wing Focus Staff"] = rolltype
    autoroller_table["Angelista's Grasp"] = rolltype
    autoroller_table["Primalist's Linked Waistguard"] = rolltype
    autoroller_table["Epaulets of Ten Storms"] = rolltype
    autoroller_table["Pendant of the Fallen Dragon"] = rolltype
    autoroller_table["Ring of Blackrock"] = rolltype
    autoroller_table["Shimmering Geta"] = rolltype
    autoroller_table["Taut Dragonhide Shoulders"] = rolltype
    autoroller_table["Therazane's Link"] = rolltype
    autoroller_table["Draconic Maul"] = rolltype
    autoroller_table["Essence Gatherer"] = rolltype
    autoroller_table["Braclets of Wrath"] = rolltype
    autoroller_table["Bindings of Transcendence"] = rolltype
    autoroller_table["Netherwind Bindings"] = rolltype
    autoroller_table["Bloodfang Bracelets"] = rolltype
    autoroller_table["Dragonstalker's Bracers"] = rolltype
    autoroller_table["Stormrage Bracers"] = rolltype
    autoroller_table["Nemesis Bracers"] = rolltype
    autoroller_table["Bracers of Ten Storms"] = rolltype
    autoroller_table["Bloodfang Belt"] = rolltype
    autoroller_table["Netherwind Belt"] = rolltype
    autoroller_table["Belt of Transcendence"] = rolltype
    autoroller_table["Waistband of Wrath"] = rolltype
    autoroller_table["Nemesis Belt"] = rolltype
    autoroller_table["Stormrage Belt"] = rolltype
    autoroller_table["Dragonstalker's Belt"] = rolltype
    autoroller_table["Helm of Endless Rage"] = rolltype
    autoroller_table["Belt of Ten Storms"] = rolltype
    autoroller_table["Red Dragonscale Protector"] = rolltype
    autoroller_table["Sabatons of Wrath"] = rolltype
    autoroller_table["Netherwind Boots"] = rolltype
    autoroller_table["Dragonstalker's Greaves"] = rolltype
    autoroller_table["Nemesis Boots"] = rolltype
    autoroller_table["Boots of Transcendence"] = rolltype
    autoroller_table["Bloodfang Boots"] = rolltype
    autoroller_table["Lifegiving Gem"] = rolltype
    autoroller_table["Stormrage Boots"] = rolltype
    autoroller_table["Heartstriker"] = rolltype
    autoroller_table["Greaves of Ten Storms"] = rolltype
    autoroller_table["Cloak of Firemaw"] = rolltype
    autoroller_table["Gauntlets of Wrath"] = rolltype
    autoroller_table["Handguards of Transcendence"] = rolltype
    autoroller_table["Bloodfang Gloves"] = rolltype
    autoroller_table["Drake Talon Pauldrons"] = rolltype
    autoroller_table["Netherwind Gloves"] = rolltype
    autoroller_table["Dragonstalker's Gauntlets"] = rolltype
    autoroller_table["Stormrange Handguards"] = rolltype
    autoroller_table["Taut Dragonhide Belt"] = rolltype
    autoroller_table["Nemesis Gloves"] = rolltype
    autoroller_table["Gauntlets of Ten Storms"] = rolltype
    autoroller_table["Ebony Flame Gloves"] = rolltype
    autoroller_table["Malfurion's Blessed Bulwark"] = rolltype
    autoroller_table["Dragonbreath Hand Cannon"] = rolltype
    autoroller_table["Shroud of Pure Thought"] = rolltype
    autoroller_table["Circle of Applied Force"] = rolltype
    autoroller_table["Netherwind Mantle"] = rolltype
    autoroller_table["Pauldron's of Wrath"] = rolltype
    autoroller_table["Bloodfang Spaulders"] = rolltype
    autoroller_table["Dragonstalker's Spaulders"] = rolltype
    autoroller_table["Pauldron's of Transcendence"] = rolltype
    autoroller_table["Stormrage Pauldrons"] = rolltype
    autoroller_table["Elementium Threaded Cloak"] = rolltype
    autoroller_table["Dragonstalker's Breastplate"] = rolltype
    autoroller_table["Netherwind Robes"] = rolltype
    autoroller_table["Robes of Transcendence"] = rolltype
    autoroller_table["Breastplate of Wrath"] = rolltype
    autoroller_table["Nemesis Robes"] = rolltype
    autoroller_table["Prestor's Talisman of Connivery"] = rolltype
    autoroller_table["Stormrage Chestguard"] = rolltype
    autoroller_table["Bloodfang Chestpiece"] = rolltype
    autoroller_table["Breastplate of the Ten Storms"] = rolltype
    autoroller_table["Cloak of Draconic Might"] = rolltype
    autoroller_table["Band of Dark Dominion"] = rolltype
    autoroller_table["Draconic Avenger"] = rolltype
    autoroller_table["Dragonfang Blade"] = rolltype
    autoroller_table["Bracers of Arcane Accuracy"] = rolltype
    autoroller_table["Firemaw's Clutch"] = rolltype
    autoroller_table["Drake Talon Cleaver"] = rolltype
    autoroller_table["Band of Forced Concentration"] = rolltype
    autoroller_table["Styleen's Impending Scarab"] = rolltype
    autoroller_table["Empowered Leggings"] = rolltype
    autoroller_table["Elementium Reinforced Bulwark"] = rolltype
    autoroller_table["Archimtiros' Ring of Reckoning"] = rolltype
    autoroller_table["Cloak of the Brood Lord"] = rolltype
    autoroller_table["Boots of the Shadow Flame"] = rolltype
    autoroller_table["Head of Nefarian"] = rolltype
    autoroller_table["Boots of Pure Thought"] = rolltype
    autoroller_table["Doom's Edge"] = rolltype
    autoroller_table["Ringo's Blizzard Boots"] = rolltype
    autoroller_table["Mantle of the Blackwing Cabal"] = rolltype
    autoroller_table["Chromatic Boots"] = rolltype
    autoroller_table["Claw of Chromaggus"] = rolltype
    autoroller_table["Pure Elementium Band"] = rolltype
    autoroller_table["The Untamed Blade"] = rolltype
    autoroller_table["Mind Quickening Gem"] = rolltype
    autoroller_table["Maladath, Runed Blade of the Black Flight"] = rolltype
    autoroller_table["Rejuvinating Gem"] = rolltype
    autoroller_table["Drake Fang Talisman"] = rolltype
    autoroller_table["Ashjre'thul, Crossbow of Smiting"] = rolltype
    autoroller_table["Chromatically Tempered Sword"] = rolltype
    autoroller_table["Neltharion's Tear"] = rolltype
    autoroller_table["Mish'undare, Circlet of the Mind Flayer"] = rolltype
    autoroller_table["Lok'amir il Romathis"] = rolltype
    autoroller_table["Staff of the Shadow Flame"] = rolltype
    autoroller_table["Ashkandi, Greatsword of the Brotherhood"] = rolltype
    autoroller_table["Crul'sorukh, Edge of Chaos"] = rolltype
    autoroller_table["Elementium Ore"] = rolltype
    print("Now rolling " .. rolltype .. " on all BWL items")
end

local function setup_aq40all(rolltype)
    autoroller_table["Anubisath Warhammer"] = rolltype
    autoroller_table["Ritssyn's Ring of Chaos"] = rolltype
    autoroller_table["Gloves of the Immortal"] = rolltype
    autoroller_table["Garb of Royal Ascension"] = rolltype
    autoroller_table["Neretzek, The Blood Drinker"] = rolltype
    autoroller_table["Gloves of the Redeemed Prophecy"] = rolltype
    autoroller_table["Shard of the Fallen Star"] = rolltype
    autoroller_table["Qiraji Bindings of Command"] = rolltype
    autoroller_table["Qiraji Bindings of Dominance"] = rolltype
    autoroller_table["Vek'lor's Diadem"] = rolltype
    autoroller_table["Vek'nilash's Circlet"] = rolltype
    autoroller_table["Ouro's Intact Hide"] = rolltype
    autoroller_table["Skin of the Great Sandworm"] = rolltype
    autoroller_table["Carapace of the Old God"] = rolltype
    autoroller_table["Husk of the Old God"] = rolltype
    autoroller_table["Imperial Qiraji Armaments"] = rolltype
    autoroller_table["Imperial Qiraji Regalia"] = rolltype
    autoroller_table["Staff of the Qiraji Prophets"] = rolltype
    autoroller_table["Leggings of Immersion"] = rolltype
    autoroller_table["Barrage Shoulders"] = rolltype
    autoroller_table["Pendant of the Qiraji Guardian"] = rolltype
    autoroller_table["Cloak of Concentrated Hatred"] = rolltype
    autoroller_table["Hammer of Ji'zhi"] = rolltype
    autoroller_table["Boots of the Redeemed Prophecy"] = rolltype
    autoroller_table["Boots of the Unwavering Will"] = rolltype
    autoroller_table["Ring of Swarming Thought"] = rolltype
    autoroller_table["Beetle Scaled Wristguards"] = rolltype
    autoroller_table["Breastplate of Annihilation"] = rolltype
    autoroller_table["Amulet of Foul Warding"] = rolltype
    autoroller_table["Boots of the Fallen Hero"] = rolltype
    autoroller_table["Gloves of Ebru"] = rolltype
    autoroller_table["Angelista's Charm"] = rolltype
    autoroller_table["Ooze-ridden Gauntlets"] = rolltype
    autoroller_table["Bile-covered Gauntlets"] = rolltype
    autoroller_table["Mantle of the Desert Crusade"] = rolltype
    autoroller_table["Mantle of the Desert's Fury"] = rolltype
    autoroller_table["Mantle of Phrenic Power"] = rolltype
    autoroller_table["Ukko's Ring of Darkness"] = rolltype
    autoroller_table["Wand of Qiraji Nobility"] = rolltype
    autoroller_table["Vest of the Swift Execution"] = rolltype
    autoroller_table["Ring of the Devoured"] = rolltype
    autoroller_table["Petrified Scarab"] = rolltype
    autoroller_table["Triad Girdle"] = rolltype
    autoroller_table["Guise of the Devourer"] = rolltype
    autoroller_table["Ternary Mantle"] = rolltype
    autoroller_table["Angelista's Touch"] = rolltype
    autoroller_table["Robes of the Triumvirate"] = rolltype
    autoroller_table["Cape of the Trinity"] = rolltype
    autoroller_table["Recomposed Boots"] = rolltype
    autoroller_table["Sartura's Might"] = rolltype
    autoroller_table["Legplates of Blazing Light"] = rolltype
    autoroller_table["Creeping Vine Helm"] = rolltype
    autoroller_table["Badge of the Swarmguard"] = rolltype
    autoroller_table["Gloves of Enforcement"] = rolltype
    autoroller_table["Robes of the Battleguard"] = rolltype
    autoroller_table["Silithid Claw"] = rolltype
    autoroller_table["Gauntlets of Steadfast Determination"] = rolltype
    autoroller_table["Thick Qirajihide Belt"] = rolltype
    autoroller_table["Leggings of the Festering Swarm"] = rolltype
    autoroller_table["Necklace of Purity"] = rolltype
    autoroller_table["Cloak of Untold Secrets"] = rolltype
    autoroller_table["Barb of the Sand Reaver"] = rolltype
    autoroller_table["Pauldrons of the Unrelenting"] = rolltype
    autoroller_table["Hive Tunneler's Boots"] = rolltype
    autoroller_table["Fetish of the Sand Reaver"] = rolltype
    autoroller_table["Ancient Qiraji Ripper"] = rolltype
    autoroller_table["Silithid Carapace Chestguard"] = rolltype
    autoroller_table["Robes of the Guardian Saint"] = rolltype
    autoroller_table["Barbed Choker"] = rolltype
    autoroller_table["Mantle of Wicked Revenge"] = rolltype
    autoroller_table["Libram of Grace"] = rolltype
    autoroller_table["Scaled Sand Reaver Leggings"] = rolltype
    autoroller_table["Sharpened Silithid Femur"] = rolltype
    autoroller_table["Gauntlets of the Righteous Champion"] = rolltype
    autoroller_table["Gauntlets of Kalimdor"] = rolltype
    autoroller_table["Scarab Brooch"] = rolltype
    autoroller_table["Ring of the Qiraji Fury"] = rolltype
    autoroller_table["Idol of Health"] = rolltype
    autoroller_table["Slime-coated Leggings"] = rolltype
    autoroller_table["Huhuran's Stinger"] = rolltype
    autoroller_table["Wasphide Gauntlets"] = rolltype
    autoroller_table["Hive Defiler Wristguards"] = rolltype
    autoroller_table["Gloves of the Messiah"] = rolltype
    autoroller_table["Ring of the Martyr"] = rolltype
    autoroller_table["Cloak of the Golden Hive"] = rolltype
    autoroller_table["Royal Qiraji Belt"] = rolltype
    autoroller_table["Royal Scepter of Vek'lor"] = rolltype
    autoroller_table["Vek'lor's Gloves of Devastation"] = rolltype
    autoroller_table["Boots of Epiphany"] = rolltype
    autoroller_table["Ring of Emperor Vek'lor"] = rolltype
    autoroller_table["Qiraji Execution Bracers"] = rolltype
    autoroller_table["Bracelets of Royal Redemption"] = rolltype
    autoroller_table["Gloves of the Hidden Temple"] = rolltype
    autoroller_table["Belt of the Fallen Emperor"] = rolltype
    autoroller_table["Amulet of Vek'nilash"] = rolltype
    autoroller_table["Grasp of the Fallen Emperor"] = rolltype
    autoroller_table["Regenerating Belt of Vek'nilash"] = rolltype
    autoroller_table["Kalimdor's Revenge"] = rolltype
    autoroller_table["Wormscale Blocker"] = rolltype
    autoroller_table["Burrower Bracers"] = rolltype
    autoroller_table["Don Rigoberto's Lost Hat"] = rolltype
    autoroller_table["Larvae of the Great Worm"] = rolltype
    autoroller_table["The Burrower's Shell"] = rolltype
    autoroller_table["Jom Gabbar"] = rolltype
    autoroller_table["Eye of C'thun"] = rolltype
    autoroller_table["Death's Sting"] = rolltype
    autoroller_table["Dark Edge of Insanity"] = rolltype
    autoroller_table["Vanquished Tentacle of C'thun"] = rolltype
    autoroller_table["Gauntlets of Annihilation"] = rolltype
    autoroller_table["Grasp of the Old God"] = rolltype
    autoroller_table["Cloak of Clarity"] = rolltype
    autoroller_table["Dark Storm Gauntlets"] = rolltype
    autoroller_table["Belt of Never-ending Agony"] = rolltype
    autoroller_table["Ring of the Godslayer"] = rolltype
    autoroller_table["Scepter of the False Prophet"] = rolltype
    autoroller_table["Eyestalk Waist Cord"] = rolltype
    autoroller_table["Cloak of the Devoured"] = rolltype
    autoroller_table["Mark of C'thun"] = rolltype
    print("Now rolling " .. rolltype .. " on all AQ40 items")
end


local function setup_mctrash(rolltype)
    autoroller_table["Lava Core"] = rolltype
    autoroller_table["Fiery Core"] = rolltype
    autoroller_table["Blood of the Mountain"] = rolltype
end

local function setup_aq20(rolltype)
    -- This is a hack. Just zero out AQ20 before doing anything.
    -- I am lazy, but this works
    autoroller_table["Gold Scarab"] = nil
    autoroller_table["Bronze Scarab"] = nil
    autoroller_table["Bone Scarab"] = nil
    autoroller_table["Stone Scarab"] = nil
    autoroller_table["Crystal Scarab"] = nil
    autoroller_table["Silver Scarab"] = nil
    autoroller_table["Clay Scarab"] = nil
    autoroller_table["Ivory Scarab"] = nil
    autoroller_table["Onyx Idol"] = nil
    autoroller_table["Jasper Idol"] = nil
    autoroller_table["Amber Idol"] = nil
    autoroller_table["Lambent Idol"] = nil
    autoroller_table["Alabaster Idol"] = nil
    autoroller_table["Azure Idol"] = nil
    autoroller_table["Obsidian Idol"] = nil
    autoroller_table["Vermillion Idol"] = nil

    if rolltype == "needclass" then
        new_roll = "need"
        _, classname, _ = UnitClass("player")
    elseif rolltype == "greedclass" then
        new_roll = "greed"
        _, classname, _ = UnitClass("player")
    elseif rolltype == "remove" then
        print("Removing rolling on all AQ20 idol and scarab items")
        rolltype = nil
        new_roll = nil
        classname = "All"
    elseif rolltype ~= "remove" then
        print("Rolling " .. rolltype .. " on ALL AQ20 idol and scarab items")
        print("This is ALL items, doesn't take your class into account")
        classname = "All"
        new_roll = rolltype
    end

    autoroller_table["Gold Scarab"] = new_roll
    autoroller_table["Bronze Scarab"] = new_roll
    autoroller_table["Bone Scarab"] = new_roll
    autoroller_table["Stone Scarab"] = new_roll
    autoroller_table["Crystal Scarab"] = new_roll
    autoroller_table["Silver Scarab"] = new_roll
    autoroller_table["Clay Scarab"] = new_roll
    autoroller_table["Ivory Scarab"] = new_roll

    if classname == "ROGUE" or classname == "WARLOCK" or classname == "WARRIOR" or classname == "All" then
        autoroller_table["Onyx Idol"] = new_roll
    end
    if classname == "DRUID" or classname == "PRIEST" or classname == "WARLOCK" or classname == "All" then
        autoroller_table["Jasper Idol"] = new_roll
    end
    if classname == "HUNTER" or classname == "PALADIN" or classname == "WARLOCK" or classname == "SHAMAN" or classname == "All" then
        autoroller_table["Amber Idol"] = new_roll
    end
    if classname == "HUNTER" or classname == "PRIEST" or classname == "WARRIOR" or classname == "All" then
        autoroller_table["Lambent Idol"] = new_roll
    end
    if classname == "DRUID" or classname == "MAGE" or classname == "WARRIOR" or classname == "All" then
        autoroller_table["Alabaster Idol"] = new_roll
    end
    if classname == "HUNTER" or classname == "MAGE" or classname == "ROGUE" or classname == "All" then
        autoroller_table["Azure Idol"] = new_roll
    end
    if classname == "MAGE" or classname == "PALADIN" or classname == "SHAMAN" or classname == "PRIEST" or classname == "All" then
        autoroller_table["Obsidian Idol"] = new_roll
    end
    if classname == "DRUID" or classname == "PALADIN" or classname == "SHAMAN" or classname == "ROGUE" or classname == "All" then
        autoroller_table["Vermillion Idol"] = new_roll
    end
end

local function setup_zg(rolltype)
    if rolltype ~= "remove" then
        print("Rolling " .. rolltype .. " on all ZG coins and bijous")
    end
    if rolltype == "remove" then
        print("Removing rolling on all ZG coins and bijous")
        rolltype = nil
    end

    autoroller_table["Bloodscalp Coin"] = rolltype
    autoroller_table["Gurubashi Coin"] = rolltype
    autoroller_table["Hakkari Coin"] = rolltype
    autoroller_table["Razzashi Coin"] = rolltype
    autoroller_table["Sandfury Coin"] = rolltype
    autoroller_table["Skullsplitter Coin"] = rolltype
    autoroller_table["Vilebranch Coin"] = rolltype
    autoroller_table["Witherbark Coin"] = rolltype
    autoroller_table["Zulian Coin"] = rolltype
    autoroller_table["Blue Hakkari Bijou"] = rolltype
    autoroller_table["Bronze Hakkari Bijou"] = rolltype
    autoroller_table["Gold Hakkari Bijou"] = rolltype
    autoroller_table["Green Hakkari Bijou"] = rolltype
    autoroller_table["Orange Hakkari Bijou"] = rolltype
    autoroller_table["Purple Hakkari Bijou"] = rolltype
    autoroller_table["Red Hakkari Bijou"] = rolltype
    autoroller_table["Silver Hakkari Bijou"] = rolltype
    autoroller_table["Yellow Hakkari Bijou"] = rolltype
end

local function setup_sfk(rolltype)
    if rolltype ~= "remove" then
        print("Rolling " .. rolltype .. " on all SFK boss drops")
    end
    if rolltype == "remove" then
        print("Removing rolling on all SFK boss drops")
        rolltype = nil
    end

    autoroller_table["Butcher's Cleaver"] = rolltype
    autoroller_table["Silverlaine's Family Seal"] = rolltype
    autoroller_table["Baron's Scepter"] = rolltype
    autoroller_table["Commander's Crest"] = rolltype
    autoroller_table["Arced War Axe"] = rolltype
    autoroller_table["Odo's Ley Staff"] = rolltype
    autoroller_table["Phantom Armor"] = rolltype
    autoroller_table["Haunting Blade"] = rolltype
    autoroller_table["Rift Bracers"] = rolltype
    autoroller_table["Fenrus' Hide"] = rolltype
    autoroller_table["Black Wolf Bracers"] = rolltype
    autoroller_table["Feline Mantle"] = rolltype
    autoroller_table["Wolfmaster Cape"] = rolltype
    autoroller_table["Robes of Arugal"] = rolltype
    autoroller_table["Belt of Arugal"] = rolltype
    autoroller_table["Meteor Shard"] = rolltype
end

function frame:OnEvent(event, arg1)
    if event == "ADDON_LOADED" and arg1 == "AutoRoller" then
        -- Our saved variables are ready at this point. If there are none, both variables will set to nil and so we set defaults.
        if autoroller_on == nil then
            autoroller_on = false
        end
        if autoroller_table == nil then
            autoroller_table = {}
        end
        if autoroller_filter == nil then
            autoroller_filter = false
        end
        if autoroller_filter == true then
            ChatFrame_AddMessageEventFilter("CHAT_MSG_LOOT", dwgautoroller_chatfilter)
        end
        if autoroller_rollfilter == true then
            ChatFrame_AddMessageEventFilter("CHAT_MSG_LOOT", dwgautoroller_chatfilter_allrolls)
        end
        if autoroller_mlhelper == nil then
            autoroller_mlhelper = false
        end
        if autoroller_disenchanter == nil then
            autoroller_disenchanter = false
        end
        -- elseif event == "PLAYER_LOGOUT" then
    end
end
frame:SetScript("OnEvent", frame.OnEvent);

local function AutoRoller_Commands(msg, editbox)
    -- pattern matching that skips leading whitespace and whitespace between cmd and args
    -- any whitespace at end of args is retained
    local _, _, cmd, args = string.find(msg, "%s?(%w+)%s?(.*)")

    -- "|cff1eff00|Hitem:11754::::::::60:::::::|h[Black Diamond]|h|r"
    if args then
        possible_name = string.match(args, "%[(.*)%]")
        if possible_name ~= nil then
            item_name = possible_name
        else
            item_name = args
        end
    end

    if ((cmd == "need" or cmd == "greed" or cmd == "pass" or cmd == "remove") and (string.lower(item_name) == "zg")) then
        setup_zg(cmd)
    elseif ((cmd == "need" or cmd == "greed" or cmd == "pass" or cmd == "remove") and (string.lower(item_name) == "sfk")) then
        setup_sfk(cmd)
    elseif ((cmd == "need" or cmd == "greed" or cmd == "pass" or cmd == "remove") and ((string.lower(item_name) == "mcall") or (string.lower(item_name) == "mc")))  then
        setup_mcall(cmd)
    elseif ((cmd == "need" or cmd == "greed" or cmd == "pass" or cmd == "remove") and ((string.lower(item_name) == "bwlall") or (string.lower(item_name) == "bwl"))) then
        setup_bwlall(cmd)
    elseif ((cmd == "need" or cmd == "greed" or cmd == "pass" or cmd == "remove") and ((string.lower(item_name) == "aq40all") or (string.lower(item_name) == "aq40"))) then
        setup_aq40all(cmd)
    elseif ((cmd == "need" or cmd == "greed" or cmd == "pass" or cmd == "remove" or cmd == "needclass" or cmd == "greedclass") and (string.lower(item_name) == "aq20")) then
        setup_aq20(cmd)
    elseif cmd == "mlhelperon" and item_name then
        if item_name == "" then
            print("Master Looter Helper On: " .. UnitName("player") .. " will be set ML when bosses are targeted")
            autoroller_mlhelper = UnitName("player")
        else
            print("Master Looter Helper On: " .. item_name .. " will be set ML when bosses are targeted")
            autoroller_mlhelper = item_name
        end
    elseif cmd == "mlhelperoff" then
        print("Master Looter Helper Off")
        autoroller_mlhelper = false
    elseif cmd == "need" and item_name ~= "" then
        print("Now rolling need on " .. item_name)
        autoroller_table[item_name] = "need"
    elseif cmd == "greed" and item_name ~= "" then
        print("Now rolling greed on " .. item_name)
        autoroller_table[item_name] = "greed"
    elseif cmd == "pass" and item_name ~= "" then
        print("Now passing on " .. item_name)
        autoroller_table[item_name] = "pass"
    elseif cmd == "remove" and item_name ~= "" then
        print("Now no longer automatically rolling on " .. item_name)
        autoroller_table[item_name] = nil
    elseif cmd == "print" then
        for table_name, table_roll in pairs(autoroller_table) do
            print("Rolling " .. table_roll .. " on " .. table_name)
        end
    elseif cmd == "clear" then
        print("Deleted all items")
        autoroller_table = {}
    elseif cmd == "on" then
        if autoroller_on == true then
            print("Autoroller was already enabled")
        else
            print("Autoroller now enabled")
        end
        autoroller_on = true
    elseif cmd == "off" then
        if autoroller_on == false then
            print("Autoroller was already disabled")
        else
            print("Autoroller now disabled")
        end
        autoroller_on = false
    elseif cmd == "filteron" then
        print("Autoroller filter now enabled")
        autoroller_filter = true
        ChatFrame_AddMessageEventFilter("CHAT_MSG_LOOT", dwgautoroller_chatfilter)
    elseif cmd == "filteroff" then
        print("Autoroller filter now disabled")
        autoroller_filter = false
        ChatFrame_RemoveMessageEventFilter("CHAT_MSG_LOOT", dwgautoroller_chatfilter)
    elseif cmd == "hiderolls" then
        print("Hiding all roll text")
        autoroller_filter = true
        ChatFrame_AddMessageEventFilter("CHAT_MSG_LOOT", dwgautoroller_chatfilter_allrolls)
    elseif cmd == "showrolls" then
        print("Showing roll text")
        autoroller_filter = false
        ChatFrame_RemoveMessageEventFilter("CHAT_MSG_LOOT", dwgautoroller_chatfilter_allrolls)
    elseif cmd == "editbox" or cmd == "eb" then
        KethoEditBox_Show(serializeTable(autoroller_table))
    else
        -- If not handled above, display some sort of help message
        if autoroller_on then
            print("Autoroller active: ON")
        end
        if not autoroller_on then
            print("Autoroller active: OFF")
        end
        if autoroller_filter then
            print("Automatic Roll Filtering: ON")
        end
        if not autoroller_filter then
            print("Automatic Roll Filtering: OFF")
        end
        if autoroller_rollfilter then
            print("All Roll Filtering: ON")
        end
        if not autoroller_rollfilter then
            print("All Roll Filtering: OFF")
        end
        if autoroller_mlhelper then
            print("Master Looter Helper: ON")
        end
        if not autoroller_mlhelper then
            print("Master Looter Helper: OFF")
        end
        print("...................................")
        print("Syntax: /autoroller ( need | greed | pass | remove ) item")
        print("Syntax: /autoroller ( needclass | greedclass ) aq20")
        print("Syntax: /autoroller ( on | off | print | clear )")
        print("Syntax: /autoroller ( filteron | filteroff ) - filters roll on items automatically rolled")
        print("Syntax: /autoroller ( hiderolls| showrolls ) - filters roll on ALL items, even those not autorolled")
        print("Syntax: /autoroller mlhelperon <playername> - Turns on Master Looter Helper, with the player as the ML. With no playername, it is you")
        print("Syntax: /autoroller mlhelperoff - Turns off Master Looter Helper")
        print("Syntax: /autoroller ( editbox | eb ) - Brings up a box to edit what to roll on")
        print(".")
        print("Using zg or aq20 instead of an item means all idols/scarabs/coins/bijous")
        print(".")
        print("Using mcall/bwlall/aq40all instead of an item rolls on (almost) all items from these zones")
        print("Useful if picking up all loot and distributing at the end.")
        print(".")
        print("needclass / greedclass only works with AQ20, and takes account of the class restrictions on idols")
    end
end

SLASH_AUTOROLLER1 = '/autoroller'
SLASH_AUTOROLLER2 = '/ar'
SlashCmdList["AUTOROLLER"] = AutoRoller_Commands

local function autoroll(self, loot_id)
    local texture, item_name, count, quality = GetLootRollItemInfo(loot_id)
    if autoroller_on == true then
        for table_name, table_roll in pairs(autoroller_table) do
            if table_name == item_name then
                if table_roll == "pass" then
                    table.insert(lootid_rolled_on, loot_id)
                    RollOnLoot(loot_id, 0)
                end
                if table_roll == "need" then
                    table.insert(lootid_rolled_on, loot_id)
                    RollOnLoot(loot_id, 1)
                end
                if table_roll == "greed" then
                    table.insert(lootid_rolled_on, loot_id)
                    RollOnLoot(loot_id, 2)
                end
            end
        end
    end
end

local function confirmloot(rollid, rolltype)
    for i, v in ipairs(lootid_rolled_on) do
        if v == rollid then
            ConfirmLootRoll(rollid, rolltype)
            StaticPopup_Hide("CONFIRM_LOOT_ROLL")
            StaticPopup_Hide("LOOT_BIND_CONFIRM")
        end
    end
end

local eventFrame2 = CreateFrame("Frame", nil, UIParent)
eventFrame2:RegisterEvent("START_LOOT_ROLL")
eventFrame2:RegisterEvent("CONFIRM_LOOT_ROLL")
eventFrame2:RegisterEvent("LOOT_BIND_CONFIRM")
eventFrame2:RegisterEvent("GROUP_ROSTER_UPDATE")
eventFrame2:RegisterEvent("PLAYER_ENTERING_WORLD")
eventFrame2:RegisterEvent("ZONE_CHANGED_NEW_AREA")
eventFrame2:RegisterEvent("CHAT_MSG_LOOT")
eventFrame2:RegisterEvent("CHAT_MSG_ADDON")
eventFrame2:RegisterEvent("UNIT_TARGET")

eventFrame2:SetScript("OnEvent", function(self, event, ...)
    if event == "CONFIRM_LOOT_ROLL" then
        local rollid = select(1, ...)
        local rolltype = select(2, ...)
        confirmloot(rollid, rolltype)
    end
    if event == "LOOT_BIND_CONFIRM" then
        local my_lootslot = select(1, ...)
        local lootIcon, my_lootName, lootQuantity, currencyID, lootQuality, locked, isQuestItem, questID, isActive = GetLootSlotInfo(my_lootslot)
        for table_name, table_roll in pairs(autoroller_table) do
            if table_name == my_lootName then
                if table_roll == "need" or table_roll == "greed" then
                    local elapsed = 0
                    self:SetScript("OnUpdate", function(self, elap)
                        elapsed = elapsed + elap
                        if elapsed < 0.2 then
                            StaticPopup_Hide("LOOT_BIND_CONFIRM")
                            return
                        end
                        elapsed = 0
                        ConfirmLootSlot(my_lootslot)
                    end)
                end
            end
        end
    end
    if event == "PLAYER_ENTERING_WORLD" or event == "GROUP_ROSTER_UPDATE" then
        lootid_rolled_on = {}
        -- Check if we are the raid leader and if we are, tell other people what we are rolling
        --[[
        if GetRealZoneText() == "Zul'Gurub" and autoroller_table["Red Hakkari Bijou"] and UnitIsGroupLeader("player") then
            C_ChatInfo.SendAddonMessage("DWGAUTOLOOT", autoroller_table["Red Hakkari Bijou"], "RAID")
        end
        if GetRealZoneText() == "Shadowfang Keep" and autoroller_table["Phantom Armor"] and UnitIsGroupLeader("player") then
            C_ChatInfo.SendAddonMessage("DWGAUTOLOOT", autoroller_table["Phantom Armor"], "RAID")
        end
        if GetRealZoneText() == "Ruins of Ahn'Qiraj" and autoroller_table["Gold Scarab"] and UnitIsGroupLeader("player") then
            C_ChatInfo.SendAddonMessage("DWGAUTOLOOT", autoroller_table["Gold Scarab"], "RAID")
        end
        --]]
    end
    if event == "ZONE_CHANGED_NEW_AREA" then
        if GetRealZoneText() == "Zul'Gurub" and autoroller_table["Red Hakkari Bijou"] then
            RaidNotice_AddMessage(RaidWarningFrame, "Rolling " .. autoroller_table["Red Hakkari Bijou"] .. " on " .. GetRealZoneText() .. " items", ChatTypeInfo["SYSTEM"])
        end
        if GetRealZoneText() == "Shadowfang Keep" and autoroller_table["Phantom Armor"] then
            RaidNotice_AddMessage(RaidWarningFrame, "Rolling " .. autoroller_table["Phantom Armor"] .. " on " .. GetRealZoneText() .. " items", ChatTypeInfo["SYSTEM"])
        end
        if GetRealZoneText() == "Ruins of Ahn'Qiraj" and autoroller_table["Gold Scarab"] then
            RaidNotice_AddMessage(RaidWarningFrame, "Rolling " .. autoroller_table["Gold Scarab"] .. " on " .. GetRealZoneText() .. " items", ChatTypeInfo["SYSTEM"])
        end
    end
    if event == "START_LOOT_ROLL" then
        local loot_id = select(1, ...)
        autoroll(self, loot_id)
    end
    if event == "UNIT_TARGET" and autoroller_mlhelper and UnitIsGroupLeader("player") then
        local settingML = 0
        local reasonName = ""
        local reasonTarget = ""
        for groupindex = 1, MAX_RAID_MEMBERS do
            name, rank, subgroup, level, class, fileName, zone, online, isDead, role, isML = GetRaidRosterInfo(groupindex)
            if name then
                targetname = GetUnitName("raid" .. groupindex .. "target")
                targetclassification = UnitClassification("raid" .. groupindex .. "target")
                targetIsDead = UnitIsDead("raid" .. groupindex .. "target")
                if targetclassification == "worldboss" and not targetIsDead then
                    settingML = 1
                    reasonName = name
                    reasonTarget = targetname
                end
            end
        end
        if settingML == 0 and GetLootMethod() == "master" then
            -- print("Setting group loot as nobody is targeting a living boss")
            SetLootMethod("group")
        end
        if settingML == 1 and GetLootMethod() ~= "master" then
            -- print("Setting " .. autoroller_mlhelper .. " as master looter because " .. reasonName .. " is targeting " .. reasonTarget)
            SetLootMethod("master", autoroller_mlhelper)
        end
    end
end)

--- INIT CODE --

-- local chatreg = C_ChatInfo.RegisterAddonMessagePrefix("DWGAUTOLOOT")

